@@ -48,8 +48,7 @@ import (
 	testgrpc "google.golang.org/grpc/test/grpc_testing"
 	testpb "google.golang.org/grpc/test/grpc_testing"
 
-	_ "google.golang.org/grpc/xds/internal/balancer/clusterresolver"        // Register the "cluster_resolver_experimental" LB policy.
-	_ "google.golang.org/grpc/xds/internal/xdsclient/controller/version/v3" // Register the v3 xDS API client.
+	_ "google.golang.org/grpc/xds/internal/balancer/clusterresolver" // Register the "cluster_resolver_experimental" LB policy.
 )
 
 const (
@@ -168,14 +167,14 @@ func clientEndpointsResource(nodeID, edsServiceName string, localities []localit
 
 // TestEDS_OneLocality tests the cluster_resolver LB policy using an EDS
 // resource with one locality. The following scenarios are tested:
-// 1. Single backend. Test verifies that RPCs reach this backend.
-// 2. Add a backend. Test verifies that RPCs are roundrobined across the two
-//    backends.
-// 3. Remove one backend. Test verifies that all RPCs reach the other backend.
-// 4. Replace the backend. Test verifies that all RPCs reach the new backend.
+//  1. Single backend. Test verifies that RPCs reach this backend.
+//  2. Add a backend. Test verifies that RPCs are roundrobined across the two
+//     backends.
+//  3. Remove one backend. Test verifies that all RPCs reach the other backend.
+//  4. Replace the backend. Test verifies that all RPCs reach the new backend.
 func (s) TestEDS_OneLocality(t *testing.T) {
 	// Spin up a management server to receive xDS resources from.
-	managementServer, nodeID, bootstrapContents, _, cleanup1 := e2e.SetupManagementServer(t, nil)
+	managementServer, nodeID, bootstrapContents, _, cleanup1 := e2e.SetupManagementServer(t, e2e.ManagementServerOptions{})
 	defer cleanup1()
 
 	// Start backend servers which provide an implementation of the TestService.
@@ -193,11 +192,11 @@ func (s) TestEDS_OneLocality(t *testing.T) {
 	}
 
 	// Create an xDS client for use by the cluster_resolver LB policy.
-	client, err := xdsclient.NewWithBootstrapContentsForTesting(bootstrapContents)
+	client, close, err := xdsclient.NewWithBootstrapContentsForTesting(bootstrapContents)
 	if err != nil {
 		t.Fatalf("Failed to create xDS client: %v", err)
 	}
-	defer client.Close()
+	defer close()
 
 	// Create a manual resolver and push a service config specifying the use of
 	// the cluster_resolver LB policy with a single discovery mechanism.
@@ -262,16 +261,16 @@ func (s) TestEDS_OneLocality(t *testing.T) {
 
 // TestEDS_MultipleLocalities tests the cluster_resolver LB policy using an EDS
 // resource with multiple localities. The following scenarios are tested:
-// 1. Two localities, each with a single backend. Test verifies that RPCs are
-//    weighted roundrobined across these two backends.
-// 2. Add another locality, with a single backend. Test verifies that RPCs are
-//    weighted roundrobined across all the backends.
-// 3. Remove one locality. Test verifies that RPCs are weighted roundrobined
-//    across backends from the remaining localities.
-// 4. Add a backend to one locality. Test verifies that RPCs are weighted
-//    roundrobined across localities.
-// 5. Change the weight of one of the localities. Test verifies that RPCs are
-//    weighted roundrobined across the localities.
+//  1. Two localities, each with a single backend. Test verifies that RPCs are
+//     weighted roundrobined across these two backends.
+//  2. Add another locality, with a single backend. Test verifies that RPCs are
+//     weighted roundrobined across all the backends.
+//  3. Remove one locality. Test verifies that RPCs are weighted roundrobined
+//     across backends from the remaining localities.
+//  4. Add a backend to one locality. Test verifies that RPCs are weighted
+//     roundrobined across localities.
+//  5. Change the weight of one of the localities. Test verifies that RPCs are
+//     weighted roundrobined across the localities.
 //
 // In our LB policy tree, one of the descendents of the "cluster_resolver" LB
 // policy is the "weighted_target" LB policy which performs weighted roundrobin
@@ -280,7 +279,7 @@ func (s) TestEDS_OneLocality(t *testing.T) {
 // weighted roundrobined across them.
 func (s) TestEDS_MultipleLocalities(t *testing.T) {
 	// Spin up a management server to receive xDS resources from.
-	managementServer, nodeID, bootstrapContents, _, cleanup1 := e2e.SetupManagementServer(t, nil)
+	managementServer, nodeID, bootstrapContents, _, cleanup1 := e2e.SetupManagementServer(t, e2e.ManagementServerOptions{})
 	defer cleanup1()
 
 	// Start backend servers which provide an implementation of the TestService.
@@ -301,11 +300,11 @@ func (s) TestEDS_MultipleLocalities(t *testing.T) {
 	}
 
 	// Create an xDS client for use by the cluster_resolver LB policy.
-	client, err := xdsclient.NewWithBootstrapContentsForTesting(bootstrapContents)
+	client, close, err := xdsclient.NewWithBootstrapContentsForTesting(bootstrapContents)
 	if err != nil {
 		t.Fatalf("Failed to create xDS client: %v", err)
 	}
-	defer client.Close()
+	defer close()
 
 	// Create a manual resolver and push service config specifying the use of
 	// the cluster_resolver LB policy with a single discovery mechanism.
@@ -403,7 +402,7 @@ func (s) TestEDS_MultipleLocalities(t *testing.T) {
 // traffic is routed only to backends deemed capable of receiving traffic.
 func (s) TestEDS_EndpointsHealth(t *testing.T) {
 	// Spin up a management server to receive xDS resources from.
-	managementServer, nodeID, bootstrapContents, _, cleanup1 := e2e.SetupManagementServer(t, nil)
+	managementServer, nodeID, bootstrapContents, _, cleanup1 := e2e.SetupManagementServer(t, e2e.ManagementServerOptions{})
 	defer cleanup1()
 
 	// Start backend servers which provide an implementation of the TestService.
@@ -439,11 +438,11 @@ func (s) TestEDS_EndpointsHealth(t *testing.T) {
 	}
 
 	// Create an xDS client for use by the cluster_resolver LB policy.
-	client, err := xdsclient.NewWithBootstrapContentsForTesting(bootstrapContents)
+	client, close, err := xdsclient.NewWithBootstrapContentsForTesting(bootstrapContents)
 	if err != nil {
 		t.Fatalf("Failed to create xDS client: %v", err)
 	}
-	defer client.Close()
+	defer close()
 
 	// Create a manual resolver and push service config specifying the use of
 	// the cluster_resolver LB policy with a single discovery mechanism.
@@ -483,7 +482,7 @@ func (s) TestEDS_EndpointsHealth(t *testing.T) {
 // removed" error.
 func (s) TestEDS_EmptyUpdate(t *testing.T) {
 	// Spin up a management server to receive xDS resources from.
-	managementServer, nodeID, bootstrapContents, _, cleanup1 := e2e.SetupManagementServer(t, nil)
+	managementServer, nodeID, bootstrapContents, _, cleanup1 := e2e.SetupManagementServer(t, e2e.ManagementServerOptions{})
 	defer cleanup1()
 
 	// Start backend servers which provide an implementation of the TestService.
@@ -505,11 +504,11 @@ func (s) TestEDS_EmptyUpdate(t *testing.T) {
 	}
 
 	// Create an xDS client for use by the cluster_resolver LB policy.
-	client, err := xdsclient.NewWithBootstrapContentsForTesting(bootstrapContents)
+	client, close, err := xdsclient.NewWithBootstrapContentsForTesting(bootstrapContents)
 	if err != nil {
 		t.Fatalf("Failed to create xDS client: %v", err)
 	}
-	defer client.Close()
+	defer close()
 
 	// Create a manual resolver and push service config specifying the use of
 	// the cluster_resolver LB policy with a single discovery mechanism.
