@@ -23,6 +23,7 @@ import (
 	"path"
 	"path/filepath"
 	"runtime"
+	"sort"
 	"strings"
 	"testing"
 	"time"
@@ -237,14 +238,12 @@ func TestImportTypeparamTests(t *testing.T) {
 
 				importedObj := imported.Scope().Lookup(name)
 				got := types.ObjectString(importedObj, types.RelativeTo(imported))
-				got = sanitizeObjectString(got)
 
 				checkedObj := checked.Scope().Lookup(name)
 				if checkedObj == nil {
 					t.Fatalf("imported object %q was not type-checked", name)
 				}
 				want := types.ObjectString(checkedObj, types.RelativeTo(checked))
-				want = sanitizeObjectString(want)
 
 				if got != want {
 					t.Errorf("imported %q as %q, want %q", name, got, want)
@@ -261,20 +260,6 @@ func TestImportTypeparamTests(t *testing.T) {
 	}
 }
 
-// sanitizeObjectString removes type parameter debugging markers from an object
-// string, to normalize it for comparison.
-// TODO(rfindley): this should not be necessary.
-func sanitizeObjectString(s string) string {
-	var runes []rune
-	for _, r := range s {
-		if '₀' <= r && r < '₀'+10 {
-			continue // trim type parameter subscripts
-		}
-		runes = append(runes, r)
-	}
-	return string(runes)
-}
-
 func checkFile(t *testing.T, filename string, src []byte) *types.Package {
 	fset := token.NewFileSet()
 	f, err := goparser.ParseFile(fset, filename, src, 0)
@@ -808,6 +793,115 @@ func TestIssue57015(t *testing.T) {
 	compileAndImportPkg(t, "issue57015")
 }
 
+// This is a regression test for a failure to export a package
+// containing a specific type error.
+//
+// Though the issue and test are specific, they may be representatives
+// of class of exporter bugs on ill-typed code that we have yet to
+// flush out.
+//
+// TODO(adonovan): systematize our search for similar problems using
+// fuzz testing, and drive this test from a table of test cases
+// discovered by fuzzing.
+func TestIssue57729(t *testing.T) {
+	// The lack of a receiver causes Recv.Type=Invalid.
+	// (The type checker then treats Foo as a package-level
+	// function, inserting it into the package scope.)
+	// The exporter needs to apply the same treatment.
+	const src = `package p; func () Foo() {}`
+
+	// Parse the ill-typed input.
+	fset := token.NewFileSet()
+	f, err := goparser.ParseFile(fset, "p.go", src, 0)
+	if err != nil {
+		t.Fatalf("parse: %v", err)
+	}
+
+	// Type check it, expecting errors.
+	config := &types.Config{
+		Error: func(err error) { t.Log(err) }, // don't abort at first error
+	}
+	pkg1, _ := config.Check("p", fset, []*ast.File{f}, nil)
+
+	// Export it.
+	// (Shallowness isn't important here.)
+	data, err := IExportShallow(fset, pkg1)
+	if err != nil {
+		t.Fatalf("export: %v", err) // any failure to export is a bug
+	}
+
+	// Re-import it.
+	imports := make(map[string]*types.Package)
+	insert := func(pkg1 *types.Package, name string) { panic("unexpected insert") }
+	pkg2, err := IImportShallow(fset, imports, data, "p", insert)
+	if err != nil {
+		t.Fatalf("import: %v", err) // any failure of IExport+IImport is a bug.
+	}
+
+	// Check that Lookup("Foo") still returns something.
+	// We can't assert the type hasn't change: it has,
+	// from a method of Invalid to a standalone function.
+	hasObj1 := pkg1.Scope().Lookup("Foo") != nil
+	hasObj2 := pkg2.Scope().Lookup("Foo") != nil
+	if hasObj1 != hasObj2 {
+		t.Errorf("export+import changed Lookup('Foo')!=nil: was %t, became %t", hasObj1, hasObj2)
+	}
+}
+
+func TestIssue58296(t *testing.T) {
+	// Compiles packages c, b, and a where c imports b and b imports a,
+	// then imports c with stub *types.Packages for b and a, and checks that
+	// both a and b are in the Imports() of c.
+	//
+	// This is how go/packages can read the exportdata when NeedDeps is off.
+
+	// This package only handles gc export data.
+	needsCompiler(t, "gc")
+	testenv.NeedsGoBuild(t) // to find stdlib export data in the build cache
+
+	// On windows, we have to set the -D option for the compiler to avoid having a drive
+	// letter and an illegal ':' in the import path - just skip it (see also issue #3483).
+	if runtime.GOOS == "windows" {
+		t.Skip("avoid dealing with relative paths/drive letters on windows")
+	}
+
+	tmpdir := mktmpdir(t)
+	defer os.RemoveAll(tmpdir)
+	testoutdir := filepath.Join(tmpdir, "testdata")
+
+	apkg := filepath.Join(testoutdir, "a")
+	bpkg := filepath.Join(testoutdir, "b")
+	cpkg := filepath.Join(testoutdir, "c")
+
+	srcdir := filepath.Join("testdata", "issue58296")
+	compilePkg(t, filepath.Join(srcdir, "a"), "a.go", testoutdir, nil, apkg)
+	compilePkg(t, filepath.Join(srcdir, "b"), "b.go", testoutdir, map[string]string{apkg: filepath.Join(testoutdir, "a.o")}, bpkg)
+	compilePkg(t, filepath.Join(srcdir, "c"), "c.go", testoutdir, map[string]string{bpkg: filepath.Join(testoutdir, "b.o")}, cpkg)
+
+	// The export data reader for c cannot rely on Package.Imports
+	// being populated for a or b. (For the imports {a,b} it is unset.)
+	imports := map[string]*types.Package{
+		apkg: types.NewPackage(apkg, "a"),
+		bpkg: types.NewPackage(bpkg, "b"),
+	}
+
+	// make sure a and b are both imported by c.
+	pkg, err := Import(imports, "./c", testoutdir, nil)
+	if err != nil {
+		t.Fatal(err)
+	}
+
+	var names []string
+	for _, imp := range pkg.Imports() {
+		names = append(names, imp.Name())
+	}
+	sort.Strings(names)
+
+	if got, want := strings.Join(names, ","), "a,b"; got != want {
+		t.Errorf("got imports %v for package c. wanted %v", names, want)
+	}
+}
+
 // apkg returns the package "a" prefixed by (as a package) testoutdir
 func apkg(testoutdir string) string {
 	apkg := testoutdir + "/a"
